# Generated by Django 3.1.1 on 2020-10-22 09:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('revpayment', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='pay_deadline',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
