# Generated by Django 3.1.1 on 2020-09-16 07:51

from django.db import migrations, models
import django.db.models.deletion
from revpayment.settings import api_settings


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(api_settings.ORDER_CLASS._meta.label),
    ]

    operations = [
        migrations.CreateModel(
            name='Logistics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('logistic_provider', models.CharField(choices=[('ecpay', 'ecpay')], default='ecpay', max_length=128)),
                ('logistic_type', models.CharField(choices=[('home', 'home'), ('cvs', 'cvs')], default='cvs', max_length=128)),
                ('logistic_subtype', models.CharField(choices=[('FAMI', 'FAMI'), ('FAMIC2C', 'FAMIC2C'), ('UNIMART', 'UNIMART'), ('UNIMARTC2C', 'UNIMARTC2C'), ('HILIFE', 'HIFIFE'), ('HILIFEC2C', 'HIFIFEC2C'), ('ECAN', 'ECAN'), ('TCAT', 'TCAT')], default='UNIMARTC2C', max_length=128)),
                ('logistic_status', models.CharField(choices=[('processing', 'processing'), ('uploading', 'uploading'), ('transit', 'transit'), ('delivered', 'delivered'), ('store_arrived', 'store_arrived')], default='processing', max_length=128)),
                ('detail', models.TextField(blank=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=api_settings.ORDER_CLASS._meta.label)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
