# Generated by Django 3.1.1 on 2020-11-04 07:22

from django.db import migrations, models
import revpayment.models


class Migration(migrations.Migration):

    dependencies = [
        ('revpayment', '0002_order_pay_deadline'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentinfo',
            name='uid',
            field=models.CharField(default=revpayment.models.uuid_hex, max_length=30),
        ),
    ]
